﻿using Kivii.Timeliness.Entities;
using Kivii.Web;
using Kivii;
using System;
using System.Collections.Generic;
using System.Linq;
using Kivii.Lims.Entities;
using Kivii.Lims.Transforms;
using Kivii.Linq;
using Kivii.Timeliness;
using Kivii.Timeliness.Extensions;
using Kivii.Lims.Extensions;

namespace Jst.Lims.Timeliness
{
    public class ModelX
    {
        public RequestDto RequestDto { get; set; }
        public ReportAssignResponse ResponseDto { get; set; }
        public Dictionary<string, object> Meta { get; set; }

        public bool IsRequestFilter { get; set; }

        public Injection Injection { get; set; }


    }
    public class Injection
    {
        public void RunResponseFilter() { }
    }
    public class RequestDto
    {
        public ReportStatusType StatusType { get; set; }

        public List<Guid> ReportKvids { get; set; }
        public Dictionary<string, string> Meta { get; set; }
    }
    public class ResponseDto
    {

    }
    public class Txxxxx
    {

        public void OnExecution(IRequest req, IResponse res)
        {
            var Model = new ModelX();
            if (Model.IsRequestFilter)
            {
                HttpError err = null;
                try
                {
                    ReportStatusType statusType = Model.RequestDto.StatusType;
                    List<Guid> reportKvids = Model.RequestDto.ReportKvids;
                    Dictionary<string, string> meta = Model.RequestDto.Meta;
                    if (statusType != ReportStatusType.DataEntry) { return; }
                    var nodeTitle = "数据录入";
                    var nodeType = NodeType.Node;
                    Model.Meta["NodeTitle"] = nodeTitle;
                    Model.Meta["NodeType"] = nodeType;
                    var connR = KiviiContext.GetOpenedDbConnection<Report>();
                    var reports = connR.Select<Report>(o => Sql.In(o.Kvid, reportKvids));
                    if (reports.IsNullOrEmpty()) { return; }
                    var conn = KiviiContext.GetOpenedDbConnection<Policy>();
                    var queryPolicy = conn.From<Policy>();
                    queryPolicy.Where(o => o.InternalCode == "Lims" && o.Enabled == true);
                    var policys = conn.Select(queryPolicy);
                    if (policys.IsNullOrEmpty()) { return; }
                    if (policys.Count() > 1) { throw new Exception("存在多个时效策略数据,请核查时效策略表!"); }
                    var policy = policys[0];
                    conn.InitEntityType<PolicyNode>();
                    var policyNodes = conn.Select<PolicyNode>(o => o.PolicyKvid == policy.Kvid);
                    if (policyNodes.IsNullOrEmpty()) { return; }

                    foreach (var report in reports)
                    {
                        if (report.CreateTime < DateTime.Parse("2024-08-24")) { continue; }//2023-11-06新规之前的报告就不绩效了
                        DateTime? deadTime = null;
                        if (report.DeadDate != null) { deadTime = report.DeadDate.Value.AddDays(1).AddSeconds(-1); }

                        var queryWorkGroups = connR.From<ReportItem>();
                        queryWorkGroups.Where(o => o.ReportKvid == report.Kvid);
                        queryWorkGroups.GroupBy(o => o.WorkGroupName);
                        queryWorkGroups.Select(o => o.WorkGroupName);
                        var workGroupNames = connR.Select<string>(queryWorkGroups);

                        ////////////////////////////////////验证当前报告是否有主节点和数据录入大节点///////////////////////////////////////////
                        var enableRequest = new TimelinessRequest();
                        enableRequest.InternalCode = "Lims";
                        enableRequest.NodeType = nodeType;
                        if (nodeType == NodeType.SubNode) { enableRequest.NodeSubTitle = nodeTitle; }
                        else { enableRequest.NodeTitle = nodeTitle; }
                        enableRequest.BizId = report.ReportId;
                        enableRequest.BizKvid = report.Kvid;
                        enableRequest.RelationTitle = report.SampleSourceBatchNumber;
                        if (!PolicyExtension.VerifyEnable(enableRequest, policy, policyNodes, conn)) { continue; }
                        var existRequest = new Kivii.Timeliness.TimelinessRequest();
                        existRequest.InternalCode = "Lims";
                        existRequest.NodeTitle = nodeTitle;
                        existRequest.NodeType = nodeType;
                        existRequest.RelationTitle = report.SampleSourceBatchNumber;
                        existRequest.BizId = report.ReportId;
                        existRequest.BizKvid = report.Kvid;
                        existRequest.PlanFinishTime = report.DeadDate;
                        existRequest.RealStartTime = DateTime.Now;
                        var existTime = PolicyExtension.GetPreNode(existRequest, policy, policyNodes, conn);
                        if (existTime == null)
                        {
                            foreach (var prop in report.GetType().GetProperties())
                            {
                                try
                                {
                                    var key = prop.Name;
                                    if (key == "Metadata") { continue; }
                                    var type = prop.GetType();
                                    var value = prop.GetValue(report, null);
                                    if (value == null) { continue; }
                                    existRequest.Metadata[key] = value.ToString();
                                    if (type == typeof(DateTime)) { existRequest.Metadata[key] = DateTime.Parse(value.ToString()).ToString("yyyy-MM-dd"); }
                                }
                                catch { continue; }
                            }
                            PolicyExtension.AddTimeliness(existRequest);
                        }
                        ///////////////////////////////////////////////////////////////////////////////

                        var groupNames = new List<string>();
                        var roles = KiviiContext.CurrentMember.Kvid.GetTestRoles();
                        if (!roles.IsNullOrEmpty() && !workGroupNames.IsNullOrEmpty())
                        {
                            foreach (var role in roles)
                            {
                                if (workGroupNames.Contains(role))
                                {
                                    Console.WriteLine("---------------------groupNames:" + role + "--------------------------");
                                    groupNames.Add(role);
                                }
                            }
                        }
                        else
                        {
                            groupNames.Add(KiviiContext.CurrentMember.DepartmentName);
                        }
                        Model.Meta[report.ReportId + "groupNames"] = groupNames;
                        foreach (var role in groupNames)
                        {
                            //再获取当前登陆人所在组的信息
                            nodeTitle = role;//KiviiContext.CurrentMember.DepartmentName;
                            nodeType = NodeType.SubNode;
                            Model.Meta["NodeTitle"] = nodeTitle;
                            Model.Meta["NodeType"] = nodeType;

                            existRequest.InternalCode = "Lims";
                            existRequest.NodeTitle = nodeTitle;
                            existRequest.NodeType = nodeType;
                            existRequest.RelationTitle = report.SampleSourceBatchNumber;
                            existRequest.BizId = report.ReportId;
                            existRequest.BizKvid = report.Kvid;
                            existRequest.PlanFinishTime = report.DeadDate;
                            existRequest.RealStartTime = DateTime.Now;
                            existTime = PolicyExtension.GetPreNode(existRequest, policy, policyNodes, conn);

                            if (existTime != null)//原本就有时效
                            {
                                Model.Meta[report.ReportId + nodeTitle] = existTime;
                            }
                            else
                            {
                                var verifyRequest = new TimelinessRequest();
                                verifyRequest.InternalCode = "Lims";
                                verifyRequest.NodeType = nodeType;
                                if (nodeType == NodeType.SubNode) { verifyRequest.NodeSubTitle = nodeTitle; }
                                else { verifyRequest.NodeTitle = nodeTitle; }
                                verifyRequest.BizId = report.ReportId;
                                verifyRequest.BizKvid = report.Kvid;
                                verifyRequest.RelationTitle = report.SampleSourceBatchNumber;
                                var verify = PolicyExtension.VerifyTimeline(verifyRequest, policy, policyNodes, conn);//验证是否超时
                                Console.WriteLine("---------------------verify:" + verify + "--------------------------");
                                var addRequest = new Kivii.Timeliness.TimelinessRequest();
                                addRequest.InternalCode = "Lims";
                                addRequest.NodeType = nodeType;
                                if (nodeType == NodeType.SubNode) { addRequest.NodeSubTitle = nodeTitle; }
                                else { addRequest.NodeTitle = nodeTitle; }
                                addRequest.RelationTitle = report.SampleSourceBatchNumber;
                                addRequest.BizId = report.ReportId;
                                addRequest.BizKvid = report.Kvid;
                                addRequest.PlanFinishTime = deadTime;
                                addRequest.Metadata = new Dictionary<string, string>();
                                foreach (var prop in report.GetType().GetProperties())
                                {
                                    try
                                    {
                                        var key = prop.Name;
                                        if (key == "Metadata") { continue; }
                                        var type = prop.GetType();
                                        var value = prop.GetValue(report, null);
                                        if (value == null) { continue; }
                                        addRequest.Metadata[key] = value.ToString();
                                        if (type == typeof(DateTime)) { addRequest.Metadata[key] = DateTime.Parse(value.ToString()).ToString("yyyy-MM-dd"); }
                                    }
                                    catch { continue; }
                                }
                                if (verify)
                                {

                                    if (!meta.IsNullOrEmpty() && !meta["Timeline"].IsNullOrEmpty())
                                    {
                                        addRequest.Summary = meta["Timeline"];
                                    }
                                    else
                                    {
                                        addRequest.Summary = "当前时效节点超时";
                                    }
                                }
                                Console.WriteLine("---------------------Request:addRequest--------------------------");
                                Model.Meta[report.ReportId + nodeTitle + "Add"] = addRequest;//有此Key说明要创建时效
                            }
                        }
                    }

                    Model.Injection.RunResponseFilter();
                    return;
                }
                catch { }
                finally
                {
                    if (err != null)
                    {
                        throw err;
                    }
                }
            }
            else
            {
                try
                {
                    ReportAssignResponse rtns = Model.ResponseDto as ReportAssignResponse;
                    if (rtns == null) { return; }
                    var th = KiviiContext.NewThread(() =>
                    {
                        foreach (var report in rtns.Results)
                        {
                            TimelinessRequest addRequest = null;
                            List<string> groupNames = null;
                            if (Model.Meta.ContainsKey(report.ReportId + "groupNames"))
                            {
                                groupNames = Model.Meta[report.ReportId + "groupNames"] as List<string>;
                            }
                            if (!groupNames.IsNullOrEmpty())
                            {
                                foreach (var role in groupNames)
                                {
                                    addRequest = null;
                                    if (Model.Meta.ContainsKey(report.ReportId + role + "Add"))
                                    {
                                        addRequest = Model.Meta[report.ReportId + role + "Add"] as TimelinessRequest;
                                    }
                                    if (addRequest != null)
                                    {
                                        PolicyExtension.AddTimeliness(addRequest);
                                        Console.WriteLine("---------------------Response:addRequest--------------------------");
                                    }
                                }
                            }
                            else { continue; }
                        }
                    });

                    th.Start();
                    return;
                }
                catch { }
            }

            return;
        }
    }
}
