﻿using Kivii;
using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Lims.Extensions;
using Kivii.Timeliness.Entities;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Jst.Lims.Timeliness
{
    [Api(Description = "时效统计")]
    public class TimelineStatistic
    {
        public int SortId { get; set; }
        public Guid Kvid { get; set; }
        public Guid PolicyNodeKvid { get; set; }

        public Guid RootKvid { get; set; }

        public Guid ParentKvid { get; set; }

        public string OperatorName { get; set; }

        public string OwnerName { get; set; }

        public Guid OwnerKvid { get; set; }//当前节点所属部门

        public string ReportId { get; set; }//报告信息
        public Guid ReportKvid { get; set; }//报告信息

        public string Title { get; set; }//节点信息

        public string Summary { get; set; }

        public string Remark { get; set; }

        public TimeFilter TimeFilter { get; set; }//时间筛选

        public decimal ReportTotalQuantity { get; set; }
        public decimal ReportQuantity { get; set; }
        public List<Report> Reports { get; set; }
        public decimal ReportNoTimelinesQuantity { get; set; }
        public List<Report> ReportNoTimelines { get; set; }
        public decimal TimelinesQuantity { get; set; }
        public List<Timeline> Timelines { get; set; }
        public decimal TimelineTimeoutsQuantity { get; set; }
        public List<Timeline> TimelineTimeouts { get; set; }
        public decimal Quantity { get; set; }
        public decimal TimeoutQuantity { get; set; }
        public decimal NormalQuantity { get; set; }
        public decimal TimeFilterQuantity { get; set; }
        public decimal TimeFiltertTimeoutQuantity { get; set; }

        public DateTime? PlanStartTime { get; set; }
        public DateTime? RealStartTime { get; set; }
        public DateTime? PlanFinishTime { get; set; }
        public DateTime? RealFinishTime { get; set; }
        public DateTime? ReportDealDate { get; set; }
        public DateTime? ReportDeadDate { get; set; }
        public DateTime? ReportPreparationDate { get; set; }//编制时间
        public DateTime? ReportReviewDate { get; set; }//审核时间
        public DateTime? ReportIssueDate { get; set; }//签发时间

        public decimal? ReportUrgentRate { get; set; }

        public List<TimelineStatistic> Details { get; set; }



    }

    [Api(Description = "报告数据录入/报告编制时效统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class TimelineStatisticDataEntry2 : RestfulExecution<TimelineStatistic>
    {
        public List<Guid> PolicyNodeKvids { get; set; }

        /// <summary>
        /// 统计的开始日期
        /// </summary>
        public DateTime BeginDate { get; set; }

        /// <summary>
        /// 统计的结束日期
        /// </summary>
        public DateTime EndDate { get; set; }

        public TimeFilter TimeFilter { get; set; }

        public bool Grouped { get; set; }
        public bool? Urgent { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginDate == DateTime.MinValue) BeginDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndDate == DateTime.MinValue) EndDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginDate = DateTime.Parse(BeginDate.ToString("yyyy-MM-dd"));
            EndDate = DateTime.Parse(EndDate.ToString("yyyy-MM-dd"));
            if (EndDate < BeginDate) throw new Exception("查询结束日期不可小于开始日期!");
            (EndDate.Subtract(BeginDate).Days > 31).ThrowIfTrue("时间跨度不得大于31天!");
            var kvids = PolicyNodeKvids.OrderBy(o => o).Join(",");
            #region 启用缓存,20分钟内查询条件相同不重复请求数据库
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = KiviiContext.GetUrnKey($"{this.GetType().FullName}_Request_{BeginDate:G}_{EndDate:G}_{TimeFilter}_{Grouped}_{Urgent}_{kvids}");
            var cacheValue = cache.Get<RestfulQueryResponse<TimelineStatistic>>(cacheKey);
            if (cacheValue != null)
            {
                return cacheValue;
            }
            #endregion

            var rtns = new RestfulQueryResponse<TimelineStatistic>();
            rtns.Results = new List<TimelineStatistic>();

            var connL = KiviiContext.GetOpenedDbConnection<Report>();
            var conn = KiviiContext.GetOpenedDbConnection<Timeline>();

            List<ReportItem> reportItems = null;
            List<Report> reports = null;
            List<Timeline> timelines = null;
            if (connL.ConnectionString == conn.ConnectionString)//同一个数据库下的情况 用leftjoin联合查询
            {
                //connL.Close();
                connL = conn;

                var queryReport = conn.From<Report>();
                queryReport.Where(o => o.DeadDate >= BeginDate && o.DeadDate <= EndDate);
                queryReport.Select(o => new { o.ReportId, o.DeadDate, o.Kvid, o.DealDate, o.PreparationDate, o.IssueDate, o.ReviewDate, o.UrgentRate,o.CreateTime });
                if (Urgent != null)
                {
                    if (Urgent.Value) queryReport.And(o => o.UrgentRate > 1);
                    else queryReport.And(o => o.UrgentRate == 1);
                }
                reports = conn.Select(queryReport);
                //Console.WriteLine($"---------------reports:{reports.Count}--------------------");
                if (reports.IsNullOrEmpty()) return rtns;
                //根据条件查出所有报告的Kvid
                var queryReportKvid = conn.From<Report>();
                queryReportKvid.Where(o => o.DeadDate >= BeginDate && o.DeadDate <= EndDate);
                if (Urgent != null)
                {
                    if (Urgent.Value) queryReportKvid.And(o => o.UrgentRate > 1);
                    else queryReportKvid.And(o => o.UrgentRate == 1);
                }
                queryReportKvid.Select(o => o.Kvid);

                //一次性查出日期范围内的报告的项目
                var queryReportItem = conn.From<ReportItem>();
                queryReportItem.Select(o => new { o.ReportKvid, o.WorkGroupName, o.Status, o.Type });
                queryReportItem.Where(o => Sql.In(o.ReportKvid, queryReportKvid));
                reportItems = conn.Select(queryReportItem);
                reportItems = reportItems.filterReportItem();
                //Console.WriteLine($"---------------reportItems:{reportItems.Count}--------------------");

                var queryBizKvid = conn.From<Report>();
                queryBizKvid.Where(o => o.DeadDate >= BeginDate && o.DeadDate <= EndDate);
                if (Urgent != null)
                {
                    if (Urgent.Value) queryBizKvid.And(o => o.UrgentRate > 1);
                    else queryBizKvid.And(o => o.UrgentRate == 1);
                }
                queryBizKvid.Select(o => o.Kvid);
                var queryTimeline = conn.From<Timeline>();
                if (PolicyNodeKvids.IsNullOrEmpty()) queryTimeline.And<Timeline>(o => o.PreKvid == Guid.Empty && o.RootKvid == o.Kvid);//不指定节点，就查根节点
                else queryTimeline.And<Timeline>(o => Sql.In(o.PolicyNodeKvid, PolicyNodeKvids) && o.PreKvid != Guid.Empty);//指定节点，不能包含根节点
                queryTimeline.And(o => Sql.In(o.BizKvid, queryBizKvid));
                queryTimeline.Select(o => new { o.BizId, o.BizKvid, o.BizType, o.OperatorName, o.OperatorKvid, o.OwnerKvid, o.OwnerName, o.Title, o.RealFinishTime, o.RealStartTime, o.PlanFinishTime, o.PlanStartTime, o.RootKvid, o.ParentKvid, o.Kvid, o.Type, o.Summary });
                timelines = conn.Select(queryTimeline);
                if (timelines.IsNullOrEmpty()) return rtns;
                //Console.WriteLine($"---------------timelines:{timelines.Count}--------------------");


            }

            if (PolicyNodeKvids.IsNullOrEmpty())//不按照节点Group统计，按照整个报告的时效进行统计
            {
                var group = reports.GroupBy(o => o.DeadDate);
                //Console.WriteLine($"---------------group:{group.Count()}--------------------");

                foreach (var kv in group)
                {
                    DateTime? filterTime = null;
                    switch (TimeFilter)
                    {
                        case TimeFilter.Unsupported:
                            filterTime = null;
                            break;
                        case TimeFilter.TwoPM:
                            filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 14:00:00"));
                            break;
                        case TimeFilter.ThreePM:
                            filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 15:00:00"));
                            break;
                        case TimeFilter.FourPM:
                            filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 16:00:00"));
                            break;
                        case TimeFilter.FourHalfPM:
                            filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 16:30:00"));
                            break;
                        case TimeFilter.FiveHalfPM:
                            filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 17:30:00"));
                            break;
                        case TimeFilter.SixPM:
                            filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 18:00:00"));
                            break;
                        case TimeFilter.TwelvePM:
                            filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 23:59:59"));
                            break;
                        default:
                            filterTime = null;
                            break;
                    }
                    var currentReports = kv.ToList();
                    var currentTimelines = timelines.Where(o => Sql.In(o.BizKvid, currentReports.ConvertAll(p => p.Kvid))).ToList();
                    var total = new TimelineStatistic();
                    total.ReportDeadDate = kv.Key;
                    //total.ReportQuantity = currentReports.Count();
                    total.Quantity = currentTimelines.IsNullOrEmpty() ? 0 : currentTimelines.Count();
                    total.TimeFilter = TimeFilter;
                    total.Details = new List<TimelineStatistic>();
                    if (currentTimelines.IsNullOrEmpty())
                    {
                        total.ReportNoTimelines = currentReports;
                    }
                    else
                    {
                        foreach (var report in kv)
                        {
                            var item = currentTimelines.FirstOrDefault(o => o.BizKvid == report.Kvid);
                            if (item == null)
                            {
                                if (total.ReportNoTimelines == null) total.ReportNoTimelines = new List<Report>();
                                total.ReportNoTimelines.Add(report);
                            }
                            else
                            {
                                if (total.Reports == null) total.Reports = new List<Report>();
                                total.Reports.Add(report);
                                if (total.Timelines == null) total.Timelines = new List<Timeline>();
                                if (total.TimelineTimeouts == null) total.TimelineTimeouts = new List<Timeline>();

                                var realFinishTime = item.RealFinishTime;
                                if (realFinishTime == null) realFinishTime = DateTime.Now;
                                if (item.PlanFinishTime != report.DeadDate.Value.AddDays(1).AddSeconds(-1)) item.PlanFinishTime = report.DeadDate.Value.AddDays(1).AddSeconds(-1);
                                if (item.PlanFinishTime != null && realFinishTime > item.PlanFinishTime) total.TimelineTimeouts.Add(item);
                                else total.Timelines.Add(item);
                                //if (filterTime != null && item.RealFinishTime != null && item.RealFinishTime <= filterTime) total.TimeFilterQuantity++;
                                //else item.TimeoutQuantity = 1;
                            }
                        }
                    }

                    rtns.Results.Add(total);
                }
            }
            else//按照节点Group
            {
                List<Report> results = new List<Report>();
                if (reportItems.IsNullOrEmpty()) return rtns;
                var groupReportItem = reportItems.GroupBy(o => o.WorkGroupName);
                //Console.WriteLine($"---------------groupReportItem:{groupReportItem.Count()}--------------------");
                foreach (var kv in groupReportItem)
                {
                    var repItems = kv.ToList();
                    var repKvids = repItems.ConvertAll(o => o.ReportKvid).Distinct().ToList();
                    //Console.WriteLine($"---------------repKvids:{repKvids.Count()}--------------------");
                    var currentReports = reports.Where(o => repKvids.Contains(o.Kvid)).ToList();
                    //Console.WriteLine($"---------------currentReports:{kv.Key}:{currentReports.Count()}--------------------");
                    foreach (var item in currentReports)
                    {
                        var reportGp = new Report();
                        reportGp.PopulateWith(item);
                        reportGp.DeadDate = DateTime.Parse(item.DeadDate.Value.ToString("yyyy-MM-dd"));
                        reportGp.Category = kv.Key;
                        results.Add(reportGp);
                    }
                }
                //Console.WriteLine($"---------------results:{results.Count()}--------------------");

                var titles = timelines.ConvertAll(x => x.Title).Distinct().ToList();
                var group = results.GroupBy(o => new { o.DeadDate, o.Category });
                //Console.WriteLine($"---------------group:{group.Count()}--------------------");
                foreach (var kv in group)
                {

                    DateTime? filterTime = null;
                    switch (TimeFilter)
                    {
                        case TimeFilter.Unsupported:
                            filterTime = null;
                            break;
                        case TimeFilter.TwoPM:
                            filterTime = DateTime.Parse(kv.Key.DeadDate.Value.ToString("yyyy-MM-dd 14:00:00"));
                            break;
                        case TimeFilter.ThreePM:
                            filterTime = DateTime.Parse(kv.Key.DeadDate.Value.ToString("yyyy-MM-dd 15:00:00"));
                            break;
                        case TimeFilter.FourPM:
                            filterTime = DateTime.Parse(kv.Key.DeadDate.Value.ToString("yyyy-MM-dd 16:00:00"));
                            break;
                        case TimeFilter.FourHalfPM:
                            filterTime = DateTime.Parse(kv.Key.DeadDate.Value.ToString("yyyy-MM-dd 16:30:00"));
                            break;
                        case TimeFilter.FiveHalfPM:
                            filterTime = DateTime.Parse(kv.Key.DeadDate.Value.ToString("yyyy-MM-dd 17:30:00"));
                            break;
                        case TimeFilter.SixPM:
                            filterTime = DateTime.Parse(kv.Key.DeadDate.Value.ToString("yyyy-MM-dd 18:00:00"));
                            break;
                        case TimeFilter.TwelvePM:
                            filterTime = DateTime.Parse(kv.Key.DeadDate.Value.ToString("yyyy-MM-dd 23:59:59"));
                            break;
                        default:
                            filterTime = null;
                            break;
                    }

                    var currentReports = kv.ToList();
                    //Console.WriteLine($"---------------currentReports:{kv.Key.DeadDate.Value.ToString("yyyy-MM-dd")} {kv.Key.Category}: {currentReports.Count()}--------------------");
                    var reportKvids = currentReports.ConvertAll(o => o.Kvid);
                    var currtReportItems = reportItems.Where(o => o.WorkGroupName == kv.Key.Category).ToList();
                    //根据检验组分组统计
                    var reportCount = currtReportItems.ConvertAll(o => o.ReportKvid).Where(o => reportKvids.Contains(o)).Distinct().Count();
                    var reportkvids = currtReportItems.ConvertAll(o => o.ReportKvid).Where(o => reportKvids.Contains(o)).Distinct().ToList();

                    var currentTimelines = timelines.Where(o => Sql.In(o.BizKvid, reportKvids) && o.Title == kv.Key.Category).ToList();
                    var total = new TimelineStatistic();
                    total.ReportDeadDate = kv.Key.DeadDate;
                    total.Title = kv.Key.Category;
                    total.ReportTotalQuantity = currentReports.Count();
                    total.Quantity = currentTimelines.IsNullOrEmpty() ? 0 : currentTimelines.Count();
                    total.TimeFilter = TimeFilter;
                    total.Details = new List<TimelineStatistic>();
                    total.Reports = new List<Report>();
                    total.Timelines = new List<Timeline>();
                    total.TimelineTimeouts = new List<Timeline>();
                    total.ReportNoTimelines = new List<Report>();

                    if (currentTimelines.IsNullOrEmpty())
                    {
                        //Console.WriteLine($"---------------currentTimelines:IsNullOrEmpty--------------------");
                        total.ReportNoTimelines.AddRange(currentReports);
                    }
                    else
                    {
                        //Console.WriteLine($"---------------currentTimelines:{currentTimelines.Count()}--------------------");
                        foreach (var report in kv)
                        {
                            //Console.WriteLine($"---------------report:{report.ReportId}--------------------");
                            var item = currentTimelines.FirstOrDefault(o => o.BizKvid == report.Kvid && o.Title == report.Category);
                            if (item == null)
                            {
                                //Console.WriteLine($"---------------item:null--------------------");
                                total.ReportNoTimelines.Add(report);
                            }
                            else
                            {
                                //Console.WriteLine($"---------------item:{item.BizId}  {item.Title}--------------------");
                                total.Reports.Add(report);

                                var realFinishTime = item.RealFinishTime;
                                if (realFinishTime == null) realFinishTime = DateTime.Now;
                                if (filterTime == null)
                                {
                                    if (item.PlanFinishTime != report.DeadDate.Value.AddDays(1).AddSeconds(-1)) item.PlanFinishTime = report.DeadDate.Value.AddDays(1).AddSeconds(-1);
                                    if (item.PlanFinishTime != null && realFinishTime > item.PlanFinishTime) total.TimelineTimeouts.Add(item);
                                    else total.Timelines.Add(item);
                                }
                                else
                                {
                                    if (realFinishTime <= filterTime) total.Timelines.Add(item);
                                    else total.TimelineTimeouts.Add(item);
                                }
                            }
                        }
                    }

                    total.ReportQuantity = total.Reports.Count;
                    total.TimelinesQuantity = total.Timelines.Count;
                    total.TimelineTimeoutsQuantity = total.TimelineTimeouts.Count;
                    total.ReportNoTimelinesQuantity = total.ReportNoTimelines.Count;

                    if (titles.Contains(total.Title)) rtns.Results.Add(total);
                }
            }
            //按照指定顺序排序
            foreach (var rtn in rtns.Results)
            {
                if (rtn.Title == "安全组") rtn.SortId = 1;
                else if (rtn.Title == "纤维含量组") rtn.SortId = 2;
                else if (rtn.Title == "微生物组") rtn.SortId = 3;
                else if (rtn.Title == "物理组") rtn.SortId = 4;
                else if (rtn.Title == "色牢度组") rtn.SortId = 5;
                else if (rtn.Title == "开样组") rtn.SortId = 6;
                else if (rtn.Title == "功能组") rtn.SortId = 7;
                else if (rtn.Title == "轻工组") rtn.SortId = 8;
                else if (rtn.Title == "纤检组") rtn.SortId = 9;
                else if (rtn.Title == "羽绒组") rtn.SortId = 10;
                else rtn.SortId = 99;
            }
            rtns.Results = rtns.Results.OrderBy(o => o.SortId).ThenBy(o => o.ReportDealDate).ToList();
            rtns.Total = rtns.Results.Count;
            cache.Set(cacheKey, rtns, TimeSpan.FromMinutes(20));
            return rtns;
        }
    }

    [Api(Description = "报告数据录入/报告编制时效统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class TimelineStatisticDataEntry : RestfulExecution<TimelineStatistic>
    {
        public List<Guid> PolicyNodeKvids { get; set; }

        /// <summary>
        /// 统计的开始日期
        /// </summary>
        public DateTime BeginDate { get; set; }

        /// <summary>
        /// 统计的结束日期
        /// </summary>
        public DateTime EndDate { get; set; }

        public TimeFilter TimeFilter { get; set; }

        public bool Grouped { get; set; }
        public bool? Urgent { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginDate == DateTime.MinValue) BeginDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndDate == DateTime.MinValue) EndDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginDate = DateTime.Parse(BeginDate.ToString("yyyy-MM-dd"));
            EndDate = DateTime.Parse(EndDate.ToString("yyyy-MM-dd"));
            if (EndDate < BeginDate) throw new Exception("查询结束日期不可小于开始日期!");

            var rtns = new RestfulQueryResponse<TimelineStatistic>();
            rtns.Results = new List<TimelineStatistic>();

            var connL = KiviiContext.GetOpenedDbConnection<Report>();
            var conn = KiviiContext.GetOpenedDbConnection<Timeline>();

            var results = new List<TimelineStatistic>();
            if (connL.ConnectionString == conn.ConnectionString)//同一个数据库下的情况 用leftjoin联合查询
            {
                //connL.Close();
                connL = conn;

                var query = conn.From<Timeline>();
                query.LeftJoin<Report>((o, r) => o.BizKvid == r.Kvid);
                if (PolicyNodeKvids.IsNullOrEmpty()) query.Where<Timeline>(o => o.PreKvid == Guid.Empty && o.RootKvid == o.Kvid);//不指定节点，就查根节点
                else query.Where<Timeline>(o => Sql.In(o.PolicyNodeKvid, PolicyNodeKvids) && o.PreKvid != Guid.Empty);//指定节点，不能包含根节点
                query.Where<Report>(o => o.DeadDate >= BeginDate && o.DeadDate <= EndDate && Sql.In(o.TypeEx, "W", "Z"));//只查看不加急并且要求完成日期在所选范围内的时效记录
                query.And<Report>(o => !o.ReportId.ToLower().Contains("test"));
                if (Urgent != null)
                {
                    if (Urgent.Value) query.And<Report>(o => o.UrgentRate > 1);
                    else query.And<Report>(o => o.UrgentRate == 1);
                }
                query.And<Report>(o => (o.BizType != "G0" && o.BizType != "G1") || o.BizType == null);
                query.OrderBy<Timeline>(o => o.PolicyNodeKvid);
                query.ThenBy<Report>(o => o.DeadDate);
                query.Select<Timeline, Report>((o, r) => new
                {
                    o.Kvid,
                    o.RootKvid,
                    o.ParentKvid,
                    o.PolicyNodeKvid,
                    o.OperatorName,
                    o.OwnerKvid,
                    o.OwnerName,
                    r.ReportId,
                    o.Title,
                    o.PlanStartTime,
                    o.RealStartTime,
                    o.PlanFinishTime,
                    o.RealFinishTime,
                    o.Summary,
                    o.Remark,
                    ReportDealDate = r.DealDate,
                    ReportDeadDate = r.DeadDate,
                    ReportPreparationDate = r.PreparationDate,
                    ReportReviewDate = r.ReviewDate,
                    ReportIssueDate = r.IssueDate,
                    ReportUrgentRate = r.UrgentRate

                });
                results = conn.Select<TimelineStatistic>(query);

            }
            if (Grouped)
            {
                if (PolicyNodeKvids.IsNullOrEmpty())//不按照节点Group统计，按照整个报告的时效进行统计
                {
                    var group = results.GroupBy(o => o.ReportDeadDate);
                    foreach (var kv in group)
                    {
                        DateTime? filterTime = null;
                        switch (TimeFilter)
                        {
                            case TimeFilter.Unsupported:
                                filterTime = null;
                                break;
                            case TimeFilter.TwoPM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 14:00:00"));
                                break;
                            case TimeFilter.ThreePM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 15:00:00"));
                                break;
                            case TimeFilter.FourPM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 16:00:00"));
                                break;
                            case TimeFilter.FourHalfPM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 16:30:00"));
                                break;
                            case TimeFilter.FiveHalfPM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 17:30:00"));
                                break;
                            case TimeFilter.SixPM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 18:00:00"));
                                break;
                            default:
                                filterTime = null;
                                break;
                        }
                        var total = new TimelineStatistic();
                        total.ReportDeadDate = kv.Key;
                        total.Quantity = kv.Count();
                        total.TimeFilter = TimeFilter;
                        total.Details = new List<TimelineStatistic>();
                        foreach (var item in kv)
                        {
                            var realFinishTime = item.RealFinishTime;
                            if (realFinishTime == null) realFinishTime = DateTime.Now;
                            if (item.PlanFinishTime != item.ReportDeadDate.Value.AddDays(1).AddSeconds(-1)) item.PlanFinishTime = item.ReportDeadDate.Value.AddDays(1).AddSeconds(-1);
                            if (item.PlanFinishTime != null && realFinishTime > item.PlanFinishTime) total.TimeoutQuantity++;
                            else total.NormalQuantity++;
                            if (filterTime != null && item.RealFinishTime != null && item.RealFinishTime <= filterTime) total.TimeFilterQuantity++;
                            else item.TimeoutQuantity = 1;
                        }
                        total.Details = kv.ToList();
                        rtns.Results.Add(total);
                    }
                }
                else//按照节点Group
                {
                    var group = results.GroupBy(o => new { o.PolicyNodeKvid,o.ReportDeadDate, o.Title });
                    foreach (var kv in group)
                    {

                        DateTime? filterTime = null;
                        switch (TimeFilter)
                        {
                            case TimeFilter.Unsupported:
                                filterTime = null;
                                break;
                            case TimeFilter.TwoPM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 14:00:00"));
                                break;
                            case TimeFilter.ThreePM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 15:00:00"));
                                break;
                            case TimeFilter.FourPM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 16:00:00"));
                                break;
                            case TimeFilter.FourHalfPM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 16:30:00"));
                                break;
                            case TimeFilter.FiveHalfPM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 17:30:00"));
                                break;
                            case TimeFilter.SixPM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 18:00:00"));
                                break;
                            default:
                                filterTime = null;
                                break;
                        }
                        var total = new TimelineStatistic();
                        total.PolicyNodeKvid = kv.Key.PolicyNodeKvid;
                        total.ReportDeadDate = kv.Key.ReportDeadDate;
                        total.Title = kv.Key.Title;
                        total.Quantity = kv.Count();
                        total.TimeFilter = TimeFilter;
                        total.Details = new List<TimelineStatistic>();
                        foreach (var item in kv)
                                {
                                    var realFinishTime = item.RealFinishTime;
                                    if (realFinishTime == null) realFinishTime = DateTime.Now;
                            //if (item.PlanFinishTime != item.ReportDeadDate.Value.AddDays(1).AddSeconds(-1)) item.PlanFinishTime = item.ReportDeadDate.Value.AddDays(1).AddSeconds(-1);
                            if (item.PlanFinishTime != null && realFinishTime > item.PlanFinishTime) total.TimeoutQuantity++;
                            else total.NormalQuantity++;
                            if (filterTime != null && item.RealFinishTime != null && item.RealFinishTime <= filterTime) total.TimeFilterQuantity++;
                            else item.TimeoutQuantity = 1;
                        }
                        total.Details = kv.ToList();
                        rtns.Results.Add(total);
                    }
                }
            }
            else rtns.Results = results;
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    [Api(Description = "报告委托受理/开样时效统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class TimelineStatisticCommission : RestfulExecution<TimelineStatistic>
    {
        public List<Guid> PolicyNodeKvids { get; set; }

        /// <summary>
        /// 统计的开始日期
        /// </summary>
        public DateTime BeginDate { get; set; }

        /// <summary>
        /// 统计的结束日期
        /// </summary>
        public DateTime EndDate { get; set; }

        //public TimeFilter TimeFilter { get; set; }

        public bool Grouped { get; set; }
        public bool? Urgent { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginDate == DateTime.MinValue) BeginDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndDate == DateTime.MinValue) EndDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginDate = DateTime.Parse(BeginDate.ToString("yyyy-MM-dd"));
            EndDate = DateTime.Parse(EndDate.ToString("yyyy-MM-dd"));
            if (EndDate < BeginDate) throw new Exception("查询结束日期不可小于开始日期!");

            var rtns = new RestfulQueryResponse<TimelineStatistic>();
            rtns.Results = new List<TimelineStatistic>();

            var connL = KiviiContext.GetOpenedDbConnection<Report>();
            var conn = KiviiContext.GetOpenedDbConnection<Timeline>();

            var results = new List<TimelineStatistic>();
            if (connL.ConnectionString == conn.ConnectionString)//同一个数据库下的情况 用leftjoin联合查询
            {
                //connL.Close();
                connL = conn;

                var query = conn.From<Timeline>();
                query.LeftJoin<Report>((o, r) => o.BizKvid == r.Kvid);
                if (PolicyNodeKvids.IsNullOrEmpty()) query.Where<Timeline>(o => o.PreKvid == Guid.Empty && o.RootKvid == o.Kvid);//不指定节点，就查根节点
                else query.Where<Timeline>(o => Sql.In(o.PolicyNodeKvid , PolicyNodeKvids) && o.PreKvid != Guid.Empty);//指定节点，不能包含根节点
                query.Where<Report>(o => o.DealDate >= BeginDate && o.DealDate <= EndDate && Sql.In(o.TypeEx, "W", "Z"));//只查看不加急并且要求完成日期在所选范围内的时效记录
                query.And<Report>(o => !o.ReportId.ToLower().Contains("test"));
                if (Urgent != null)
                {
                    if (Urgent.Value) query.And<Report>(o => o.UrgentRate > 1);
                    else query.And<Report>(o => o.UrgentRate == 1);
                }
                query.And<Report>(o => (o.BizType != "G0" && o.BizType != "G1") || o.BizType == null);
                query.OrderBy<Timeline>(o => o.PolicyNodeKvid);
                query.ThenBy<Report>(o => o.DealDate);
                query.Select<Timeline, Report>((o, r) => new
                {
                    o.Kvid,
                    o.RootKvid,
                    o.ParentKvid,
                    o.PolicyNodeKvid,
                    o.OwnerKvid,
                    o.OwnerName,
                    o.OperatorName,
                    r.ReportId,
                    o.Title,
                    o.PlanStartTime,
                    o.RealStartTime,
                    o.PlanFinishTime,
                    o.RealFinishTime,
                    o.Summary,
                    o.Remark,
                    ReportDealDate = r.DealDate,
                    ReportDeadDate = r.DeadDate,
                    ReportPreparationDate = r.PreparationDate,
                    ReportReviewDate = r.ReviewDate,
                    ReportIssueDate = r.IssueDate,
                    ReportUrgentRate = r.UrgentRate

                });
                results = conn.Select<TimelineStatistic>(query);

            }
            if (Grouped)
            {
                if (PolicyNodeKvids.IsNullOrEmpty())//不按照节点Group统计，按照整个报告的时效进行统计
                {
                    var group = results.GroupBy(o => o.ReportDealDate);
                    foreach (var kv in group)
                    {
                        var total = new TimelineStatistic();
                        total.ReportDealDate = kv.Key;
                        total.Quantity = kv.Count();
                        total.Details = new List<TimelineStatistic>();
                        var records = kv.OrderBy(o => o.PlanFinishTime).ToList();
                        var grop = records.GroupBy(o => o.PlanFinishTime);
                        foreach (var gp in grop)
                        {
                            var detail = new TimelineStatistic();
                            detail.PlanFinishTime = gp.Key;
                            detail.Quantity = gp.Count();
                            detail.Details = new List<TimelineStatistic>();
                            foreach (var item in gp)
                            {
                                var realFinishTime = item.RealFinishTime;
                                if (realFinishTime == null) realFinishTime = DateTime.Now;
                                if (item.PlanFinishTime != null && realFinishTime > item.PlanFinishTime)
                                {
                                    detail.TimeoutQuantity++;
                                    item.TimeoutQuantity++;
                                    total.TimeoutQuantity++;
                                }
                                else
                                {
                                    detail.NormalQuantity++;
                                    item.NormalQuantity++;
                                    total.NormalQuantity++;
                                }
                                detail.Details.Add(item);
                            }
                            total.Details.Add(detail);
                        }
                        rtns.Results.Add(total);
                    }
                }
                else//按照节点Group
                {
                    var group = results.GroupBy(o => new { o.PolicyNodeKvid,o.ReportDealDate, o.Title });
                    foreach (var kv in group)
                    {
                        var total = new TimelineStatistic();
                        total.PolicyNodeKvid = kv.Key.PolicyNodeKvid;
                        total.ReportDealDate = kv.Key.ReportDealDate;
                        total.Title = kv.Key.Title;
                        total.Quantity = kv.Count();
                        total.Details = new List<TimelineStatistic>();
                        var records = kv.OrderBy(o => o.PlanFinishTime).ToList();
                        var grop = records.GroupBy(o => o.PlanFinishTime);
                        foreach (var gp in grop)
                        {
                            var detail = new TimelineStatistic();
                            detail.PlanFinishTime = gp.Key;
                            detail.Quantity = gp.Count();
                            detail.Details = new List<TimelineStatistic>();
                            foreach (var item in gp)
                            {
                                var realFinishTime = item.RealFinishTime;
                                if (realFinishTime == null) realFinishTime = DateTime.Now;
                                if (item.PlanFinishTime != null && realFinishTime > item.PlanFinishTime)
                                {
                                    detail.TimeoutQuantity++;
                                    item.TimeoutQuantity++;
                                    total.TimeoutQuantity++;
                                }
                                else
                                {
                                    detail.NormalQuantity++;
                                    item.NormalQuantity++;
                                    total.NormalQuantity++;
                                }
                                detail.Details.Add(item);
                            }
                            total.Details.Add(detail);
                        }
                        rtns.Results.Add(total);
                    }
                }
            }
            else rtns.Results = results;
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    public enum TimeFilter
    {
        Unsupported = 0,

        /// <summary>
        /// 14:00
        /// </summary>
        TwoPM,

        /// <summary>
        /// 15:00
        /// </summary>
        ThreePM,

        /// <summary>
        /// 16:00
        /// </summary>
        FourPM,

        /// <summary>
        /// 16:30
        /// </summary>
        FourHalfPM,

        /// <summary>
        /// 17:30
        /// </summary>
        FiveHalfPM,

        /// <summary>
        /// 18:00
        /// </summary>
        SixPM,

        /// <summary>
        /// 24:00
        /// </summary>
        TwelvePM
    }
}
